@@ -3,51 +3,7 @@
     Copyright 2021 Hwakyeom Kim(=just-do-halee)
 */
 
-//! First, You should make your own an error set.
-//! # Example
-//! ```no_run
-//! err! {
-//!      BrokenHeader => "broken header."
-//!      AnotherHeader => "not matched header."
-//!      FileNotFound => "file not found."
-//!      EmptyArgument => "empty argument."
-//!      UnexpectedEof => "unexpected eof."
-//!      OutOfBounds => "index out of bounds."
-//!      NotMatched => "btw not matched."
-//! }
-//! ```
-//! And just errbang!
-//! ```no_run
-//! errbang!(err::BrokenHeader);
-//! ```
-//! # More Examples
-//! ```no_run
-//! fn foo() -> Result<bool> { // Our Master Result Type
-//!     let bar = 2;
-//!     match bar {
-//!         0 => Ok(true),
-//!         1 => Ok(false),
-//!         _ => errbang!(err::NotMatched, "{} is {}", "bar", bar),
-//!     }
-//! }
-//! fn main() -> Result<()> {
-//!     let _is_bar_zero = foo()?;
-//!     Ok(())
-//! }
-//! ```
-//! Please use our Master Result<T> and ResultSend<T> instead std::result::Result or io::Result etc..
-//! ```no_run
-//! /// Master Result
-//! pub type Result<T> = result::Result<T, Box<dyn error::Error>>;
-//! /// Master Result for Send + Sync trait
-//! pub type ResultSend<T> = result::Result<T, Box<dyn error::Error + Send + Sync>>;
-//! ```
-
-//! ---
-//! ### just put this in your project.
-//! ```rust
-//! pub use utils_results::*;
-//! ```
+#[doc = include_str!("../../README.md")]
 
 /// make some error. Master Result::Err()
 /// ```no_run
@@ -132,7 +88,7 @@ macro_rules! errextract {
     };
 }
 
-/// create custom errors list
+/// create custom error list
 /// ```no_run
 /// err! {
 ///      BrokenHeader => "broken header."
@@ -148,72 +104,120 @@ macro_rules! errextract {
 /// ```
 #[macro_export]
 macro_rules! err {
+    (
+        @create errstruct $kind:ident $message:tt
+    ) => {
+
+        #[derive(Debug)]
+        pub struct $kind {
+            meta: String,
+            message: &'static str,
+        }
+
+        impl $kind {
+            pub fn new(meta: String) -> Self {
+                Self { meta, message: $message }
+            }
+            pub fn as_combination(&self) -> String {
+                format!("{} {}", self.meta, self.message)
+            }
+        }
+
+        impl std::error::Error for $kind {
+            fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+                Some(self)
+            }
+        }
+        impl std::fmt::Display for $kind {
+            fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+                write!(f, "{}", self.as_combination())
+            }
+        }
+
+    };
     (
             $($kind:ident => $message:tt$(,)?)*
     ) => {
 
         pub mod err {
+            use super::*;
+
+            err!(@create errstruct __ "external error");
+
 
             $(
-                #[derive(Debug)]
-                pub struct $kind {
-                    meta: String,
-                    message: &'static str,
-                }
-
-                impl $kind {
-                    pub fn new(meta: String) -> Self {
-                        Self { meta, message: $message }
-                    }
-                    pub fn as_combination(&self) -> String {
-                        format!("{} {}", self.meta, self.message)
-                    }
-                }
-
-                impl std::error::Error for $kind {
-                    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
-                        Some(self)
-                    }
-                }
-                impl std::fmt::Display for $kind {
-                    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-                        write!(f, "{}", self.as_combination())
-                    }
-                }
-
+                err!(@create errstruct $kind $message);
             )*
 
         }
+
     };
 }
 
-/// casting std::io Error to Master Error (custom list)
+/// matching io::Error and Master Error to use casting error macros<br>
 ///```no_run
-/// fn_handle_io_error! {
+/// io_to_err!(file.seek(SeekFrom::End(0)))?; // <- io::Error to err
+/// err_to_io!(my_seek(0))?; // <- err to io::Error
+///```
+///```no_run
+/// io_err! {
 ///     // std::io::ErrorKind => err::MyError
 ///     UnexpectedEof => err::MyError1
 ///     Interrupted => err::MyError2
 /// }
 ///```
-/// and using in anywhere
-///```rust
-/// fn_handle_io_error(file.seek(SeekFrom::End(0)))?
-///```
 #[macro_export]
-macro_rules! fn_handle_io_error {
+macro_rules! io_err {
     (
         $($kind:ident => $errkind:ty$(,)?)*
     ) => {
-        pub fn fn_handle_io_error<T>(io_error: std::io::Result<T>) -> Result<T> {
+        pub fn fn_handle_io_to_err<T>(io_error: std::io::Result<T>, meta: String) -> Result<T> {
             match io_error {
                 Err(e) => match e.kind() {
                     $(
-                        std::io::ErrorKind::UnexpectedEof => errbang!(err::UnexpectedEof),
+                        std::io::ErrorKind::$kind => Err(Box::new(<$errkind>::new(meta))),
                     )*
                     _ => Err(Box::new(e)),
                 },
                 Ok(t) => Ok(t),
             }
         }
+        pub fn fn_handle_err_to_io<T>(m_error: Result<T>) -> std::io::Result<T> {
+            match m_error {
+                Err(e) => match e {
+                    $(
+                        e if errmatch!(e, $errkind) => std::io::Result::Err(std::io::Error::new(std::io::ErrorKind::$kind, format!("{:?}", e))),
+                    )*
+                    _ => std::io::Result::Err(std::io::Error::new(std::io::ErrorKind::Other, format!("{:?}", e))),
+                },
+                Ok(t) => std::io::Result::Ok(t),
+            }
+        }
+    };
+}
+
+/// casting std::io Error to Master Error matched by `io_err`
+///```no_run
+/// io_to_err!(file.seek(SeekFrom::End(0)))?
+///```
+#[macro_export]
+macro_rules! io_to_err {
+    (
+        $ioe:expr
+    ) => {
+        fn_handle_io_to_err($ioe, format!("[{}:{}] io to err", file!(), line!()))
+    };
+}
+
+/// casting Master Error to std::io Error matched by `io_err`
+///```no_run
+/// err_to_io!(my_seek(0))?
+///```
+#[macro_export]
+macro_rules! err_to_io {
+    (
+        $err:expr
+    ) => {
+        fn_handle_err_to_io($err)
     };
 }
